@@ -15,8 +15,9 @@
 
 #define NR_CANDIDATES 10
 
-static int im_uim_fd;
+static int im_uim_fd = -1;
 static int read_tag;
+static char *result = NULL;
 
 struct preedit_segment {
   int attr;
@@ -47,6 +48,7 @@ typedef struct _IMUIMContext {
   GtkWidget *stat_win;
   GtkWidget *menu;
   GdkWindow *win;
+  GdkWindow *toplevel;
   GdkRectangle preedit_pos; /* preedit_pos not always point the cursor location */
   int mode;
   /**/
@@ -54,7 +56,7 @@ typedef struct _IMUIMContext {
 } IMUIMContext;
 
 static IMUIMContext context_list;
-
+static IMUIMContext *focused_context = NULL;
 static GObjectClass *parent_class;
 
 typedef struct _IMContextUIMClass 
@@ -69,20 +71,25 @@ static void show_preedit(GtkIMContext *ic, GtkWidget *preedit_label);
 static void helper_disconnect_cb(void);
 static void helper_read_cb(gpointer p, int fd, GdkInputCondition c);
 
+static void
+im_uim_parse_helper_str(char *str);
+
 static const GTypeInfo class_info = {
   sizeof(IMContextUIMClass),
   (GBaseInitFunc) NULL,
   (GBaseFinalizeFunc) NULL,
   (GClassInitFunc) im_uim_class_init,
-  NULL,
-  NULL,
-  sizeof(IMUIMContext),
+  (GClassFinalizeFunc)NULL,
+  NULL,/*for class data*/
+  sizeof(IMUIMContext), /* size of instance */
   0,
-  (GtkObjectInitFunc) NULL,
+  (GInstanceInitFunc)NULL, /* constructor */
 };
 
-static GType type_im_uim;
-#define IM_UIM_CONTEXT(obj) (GTK_CHECK_CAST((obj),type_im_uim,IMUIMContext))
+static GType type_im_uim = 0;
+
+#define IM_UIM_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),type_im_uim,IMUIMContext))
+
 static const GtkIMContextInfo im_uim_info = {
   "uim",
   "UIM()",
@@ -306,7 +313,6 @@ im_uim_set_cursor_location(GtkIMContext *ic,
 
   uic->preedit_pos.x = area->x;
   uic->preedit_pos.x = area->x;
-
 }
 
 
@@ -337,6 +343,10 @@ void focus_in(GtkIMContext *ic)
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
   IMUIMContext *cc;
 
+  focused_context = uic;
+
+  uim_prop_list_update(uic->uc);
+
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
     if (cc != uic) {
       if (uic->stat_win) {
@@ -359,6 +369,7 @@ void focus_in(GtkIMContext *ic)
 void focus_out(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
+
   if (uic->win && uic->stat_win) {
     gtk_widget_hide(uic->stat_win);
   }
@@ -411,8 +422,7 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
   gtk_im_context_get_preedit_string (ic, &str, &attrs, &cursor_pos);
 
   
-  if ( strlen (str) > 0)
-    {
+  if ( strlen (str) > 0) {
       gint x,y,w,h;
       PangoLayout *layout;
 
@@ -445,11 +455,34 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
   
 }
 
-
+/*
 static GdkFilterReturn
 resize_window_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
 {
-  IMUIMContext *uic = IM_UIM_CONTEXT(data);
+  IMUIMContext *uic;
+
+  if(!data)
+    return GDK_FILTER_CONTINUE;   
+
+  uic = IM_UIM_CONTEXT(data);
+
+
+  return GDK_FILTER_CONTINUE;
+
+}
+*/
+
+static GdkFilterReturn
+toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
+{
+  IMUIMContext *uic;
+
+  uic = IM_UIM_CONTEXT(data);
+
+
+  if(uic->cwin.top_win && uic->cwin.is_active ){
+    layout_candidate(uic);
+  }
   update_status_window(uic);
   return GDK_FILTER_CONTINUE;
 }
@@ -459,18 +492,30 @@ static void
 set_client_window(GtkIMContext *ic, GdkWindow *w)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
-  if (uic->win != w) {
+
+  GdkWindow *toplevel = NULL;
+
+  if(w)
+    toplevel = gdk_window_get_toplevel(w);
+
+  if ( uic->win != w ) {
     if (uic->win) {
-      gdk_window_remove_filter(uic->win, resize_window_cb, uic);
+      /*      gdk_window_remove_filter(uic->toplevel, resize_window_cb, uic);*/
+      gdk_window_remove_filter(uic->toplevel, toplevel_window_candidate_cb, uic);
     }
-    if (w) {
-      GdkEventMask mask;
-      mask = gdk_window_get_events(w);
-      gdk_window_set_events(w, mask | GDK_STRUCTURE_MASK);
-      gdk_window_add_filter(w, resize_window_cb, uic);
+    if (w && toplevel) {
+      /*      gdk_window_add_filter(toplevel,
+			     resize_window_cb,
+			     uic);*/
+       
+       gdk_window_add_filter(toplevel,
+			     toplevel_window_candidate_cb,
+			     uic);
+       
     }
   }
   uic->win = w;
+  uic->toplevel = toplevel;
   update_status_window(uic);
 }
 
@@ -483,6 +528,7 @@ im_uim_finalize(GObject *obj)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(obj);
   set_client_window(GTK_IM_CONTEXT(uic), NULL);
+
   uic->next->prev = uic->prev;
   uic->prev->next = uic->next;
 
@@ -591,14 +637,22 @@ update_mode_list_cb(void *ptr)
 static void
 update_prop_list_cb(void *ptr, char *str)
 {
-  printf("gtk prop list callbak %s\n",str);
-  uim_helper_send_message(im_uim_fd, str);
+  GString *tmp = g_string_new("");
+  g_string_printf(tmp, "prop_list_update\ncharset=UTF-8\n%s", str);
+ 
+  uim_helper_send_message(im_uim_fd, tmp->str);
+  g_string_free(tmp, TRUE);
 }
 
 static void
 update_prop_label_cb(void *ptr, char *str)
 {
-  uim_helper_send_message(im_uim_fd, str);
+  GString *tmp = g_string_new("");
+  g_string_printf(tmp, "prop_label_update\ncharset=UTF-8\n%s", str);
+  /*  g_print("label %s\n",tmp->str);*/
+
+  uim_helper_send_message(im_uim_fd, tmp->str);
+  g_string_free(tmp, TRUE);
 }
 
 static void
@@ -722,9 +776,12 @@ layout_candidate(IMUIMContext *uic)
 
   new_first_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin);
   gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), 0, 1, &old_first_cand);
+
+  if(!new_first_cand)
+    return;
+
   /* This condition works almost case, but not perfect.
      We must think out more good way.  */
-
   if( !old_first_cand ||
       strcmp( old_first_cand, new_first_cand) !=0 ) {
 
@@ -742,9 +799,9 @@ layout_candidate(IMUIMContext *uic)
       }
       gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
 
-      if(strcmp(buf[1], "") != 0 ) {
+      if(buf[1] && strcmp(buf[1],"") != 0 )
 	free(buf[1]);
-      }
+   
     }
     gtk_clist_thaw( GTK_CLIST(uic->cwin.clist) );
   }
@@ -755,7 +812,7 @@ layout_candidate(IMUIMContext *uic)
   gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
 		       uic->cwin.candidate_index - uic->cwin.layout_begin,
 		       0);
-  sprintf(idx, "%d/%d",uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates); 
+  sprintf(idx, "%d / %d",uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates); 
 
   gtk_label_set_text( GTK_LABEL(uic->cwin.numlabel), idx);
 
@@ -791,13 +848,10 @@ cand_end_cb(void *ptr)
 }
 
 
-/*
- * CONSTRUCTOR
- */
 GtkIMContext *
 im_module_create (const gchar *context_id)
 {
-  GtkObject *obj = g_object_new(type_im_uim, NULL);
+  GObject *obj = g_object_new(type_im_uim, NULL);
   IMUIMContext *uic = IM_UIM_CONTEXT(obj);
 
   uic->win = NULL;
@@ -815,21 +869,20 @@ im_module_create (const gchar *context_id)
     context_id = &context_id[4];
   }
 
-  im_uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);
+  if(im_uim_fd < 0) {
+    im_uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);
+    if (im_uim_fd >= 0) {
+      read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
+			       helper_read_cb, 0);
+    }
 
-  if (im_uim_fd >= 0) {
-    read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
-			     helper_read_cb, 0);
   }
 
   uim_init();
 
-
   uic->uc =  uim_create_context(uic, "UTF-8",
 				NULL, (char *)context_id,
 				im_uim_commit_string);
-  /* to be a global context */
-  uim_helper_manage_context(uic->uc);
   /**/
   uim_set_mode_cb(uic->uc, mode_cb);
   uim_set_preedit_cb(uic->uc, clear_cb,
@@ -892,13 +945,50 @@ im_module_list(GtkIMContextInfo ***contexts,
   *n_contexts = nr;
 }
 
+static void
+im_uim_parse_helper_str(char *str)
+{
+  gchar **tmp;
+  gchar **tmp2;
+  int i = 0;
+
+  tmp = g_strsplit(str, "\n\n", 0);
+
+  while(tmp[i]){
+    /*    fprintf(stderr,"tmp:%d:%s\n",i,tmp[i]);*/
+  
+    if(focused_context) {
+      if(strcmp("prop_list_get", tmp[i]) == 0){
+	uim_prop_list_update(focused_context->uc);
+      } else if(strcmp("prop_label_get", tmp[i]) == 0) {
+	uim_prop_label_update(focused_context->uc);
+      } else if(strncmp("prop_activate", tmp[i], 13) == 0) {
+	tmp2 = g_strsplit(str, "\n", 0);
+	if(tmp2 && tmp2[0]) {
+	  uim_prop_activate(focused_context->uc, tmp2[1]);
+	  g_strfreev(tmp2);
+	}
+      } else if(strcmp("focus_in", tmp[i]) == 0) {
+	focused_context = NULL;
+      }
+    }
+    i++;
+  }
+  
+  g_strfreev(tmp);
+}
+
+
 static void
 helper_read_cb(gpointer p, int fd, GdkInputCondition c)
 {
   /*  uim_helper_read_proc();*/
   static char buf[1024];
-  char *result = NULL;
   int i,rc;
+
+  if(!result)
+    result = strdup("");
+
   while((i = uim_helpler_fd_readable(fd)) > 0) {
 
     rc = read(fd, buf, sizeof(buf)-1);
@@ -909,19 +999,18 @@ helper_read_cb(gpointer p, int fd, GdkInputCondition c)
     }
 
     buf[rc] = '\0';
-    printf("read: %slength %d\n",buf, strlen(buf));
-    
-    
-    if(!result) {
-      result = (char*)malloc(strlen(buf)+1); 
-      result[0] = '\0';
-    } else {
-      result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
-    }
-   
-  strcat(result, buf);
+    result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
+  
+    strcat(result, buf);
   }
-  free(result);
+
+  if(uim_helper_str_terminated(result)) {
+    im_uim_parse_helper_str(result);
+    free(result);
+    result = NULL;
+  }
+  
+
 }
 
 static void
