@@ -212,7 +212,7 @@
 	   (ac (context-data c)))
       (anthy-flush ac)
       (if (= mode 0)
-	  (anthy-context-set-on! ac '())
+	  (anthy-context-set-on! ac #f)
 	  (anthy-context-set-on! ac #t))
       (anthy-update-preedit ac id))
     ()))
@@ -251,7 +251,8 @@
       (if (anthy-on-key key key-state)
 	  (begin
 	    (anthy-begin-input ac)
-	    (im-update-mode id 1))
+	    (im-update-mode id 1)
+	    (anthy-update-prop-label ac))
 	  (im-commit-raw (context-id c))))))
 
 (define anthy-make-index-list-rec
@@ -306,6 +307,7 @@
 	     (anthy-flush ac)
 	     (anthy-context-set-on! ac #f)
 	     (im-update-mode id 0)
+	     (anthy-update-prop-label ac)
 	     #f)
 	   #t)
        (if (anthy-backspace-key key key-state)
@@ -687,34 +689,63 @@
 
 (define anthy-prop-handler
   (lambda (id message)
-    (if (= message
-	   "prop_anthy_hira")
-	())
-    (print "prop handler\n")
-	))
+    (let* ((c (find-context id))
+	   (ac (context-data c)))
+      (anthy-flush ac)
+      (anthy-update-preedit ac id)
+    (if (string-equal? message
+		       "prop_anthy_hiragana")
+	(begin
+	  (im-update-mode id 1)
+	  (anthy-context-set-on! ac #t)))
+    (if (string-equal? message
+		       "prop_anthy_katakana")
+	(begin
+	  (im-update-mode id 1)
+	  (anthy-context-set-on! ac #t)))
+    (if (string-equal? message
+		       "prop_anthy_direct")
+	(begin
+	  (im-update-mode id 0)
+	  (anthy-context-set-on! ac #f)))
+    (anthy-update-prop-label ac))))
 
 (define anthy-update-prop-label
   (lambda (ac)
-    (let ((id (anthy-context-ac-id ac)))
-    (im-update-prop-label "a\tϾ\nϥ⡼\n"))))
+    (let* ((id (anthy-context-ac-id ac))
+	   (str ""))
+      (set! str
+	    (if (anthy-context-on ac)
+		"\tҤ餬\n"
+		"a\tľ\n"))  
+      (set! str (string-append str "\tϥ⡼\n"))
+      (im-update-prop-label id str))))
   
-
 (define anthy-prop-list
   '(
     ("branch" "Ͼ")
-    ("leaf" "" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "prop_anthy_hira")
-    ("leaf" "" "" "" "prop_anthy_kana")
+    ("leaf" "" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "prop_anthy_hiragana")
+    ("leaf" "" "" "" "prop_anthy_katakana")
     ("leaf" "" "ѱѿ" "ѱѿ⡼" "prop_anthy_zenkaku")
-    ("leaf" "a" "ľ" "ե٥åȤϤǤޤ" "prop_anthy_direct")    
+    ("leaf" "a" "ľ" "ե٥åȤϤǤޤ" "prop_anthy_direct")
     ("branch" "ϥ⡼")
     ("leaf" "" "޻" "޻" "prop_anthy_roma")
     ("leaf" "" "" "" "prop_anthy_kana")
     ))
 
 (define anthy-update-prop-list
-  (lambda (id)
-    (im-update-prop-list id "branch\tϾ\nleaf\t\tҤ餬\tdesc\tprop_anthy_hira\nbranch\t\tR\t޻\tdesc\tprop_anthy_roma\n")
-    ))
+  (lambda (id) 
+    (let* ((c (find-context id))
+	   (ac (context-data c))
+	   (str "branch\t"))
+      (set! str (string-append str
+	    (if (anthy-context-on ac)
+		"\tҤ餬\n"
+		"a\tľ\n")))
+      (set! str (string-append str "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_anthy_hiragana\nleaf\t\t\tʤϤǤޤ\tprop_anthy_katakana\nleaf\tA\tľ\t쥯ȤǤ\tprop_anthy_direct\nbranch\t\t\nleaf\t\t޻\tdesc\tprop_anthy_roma\nleaf\t\t\t\tdescription\tprop_anthy_kana\n"))
+    (im-update-prop-list id str)
+    
+    )))
 
 (register-im
  'anthy
