@@ -704,7 +704,7 @@
 
 (define prime-proc-mode-wide-latin
   (lambda (c key key-state)
-    (let* ((w (jp-wide (charcode->string key)))
+    (let* ((w (ja-wide (charcode->string key)))
 		   (id (context-id c))
 		   (sc (context-data c)))
       (if (prime-on-key key key-state)
