@@ -17,6 +17,8 @@
 (define skk-dic-file-name "/usr/share/skk/SKK-JISYO.L")
 (define skk-personal-dic-filename
   (string-append (getenv "HOME") "/.skk-jisyo"))
+(define skk-uim-personal-dic-filename
+  (string-append (getenv "HOME") "/.skk-uim-jisyo"))
 (define skk-dic-init nil)
 ;; configs
 (define skk-use-candidate-window? #t)
@@ -177,6 +179,15 @@
 	  (skk-find-descendant-context csc)
 	  sc))))
 
+(define skk-read-personal-dictionary
+  (lambda ()
+    (or (skk-lib-read-personal-dictionary skk-uim-personal-dic-filename)
+	(skk-lib-read-personal-dictionary skk-personal-dic-filename))))
+
+(define skk-save-personal-dictionary
+  (lambda ()
+    (skk-lib-save-personal-dictionary skk-uim-personal-dic-filename)))
+
 (define skk-current-context
   (lambda (c)
     (skk-find-descendant-context (context-data c))))
@@ -201,7 +212,7 @@
 	  (if skk-use-recursive-learning?
 	   (require "skk-editor.scm"))
 	  (skk-lib-dic-open skk-dic-file-name)
-	  (skk-lib-read-personal-dictionary skk-personal-dic-filename)))
+	  (skk-read-personal-dictionary)))
     (let ((sc 
 	   (copy-list 
 	    '(skk-state-latin #t "" "" "" () () () () () () () () ()))))
@@ -599,10 +610,11 @@
     (if (not (skk-get-current-candidate sc))
 	(begin
 	  (skk-context-set-nth! sc 0)
-	  (if (skk-context-candidate-window sc)
-	      (im-end-candidate id))
 	  (if skk-use-recursive-learning?
-	      (skk-setup-child-context sc id))))
+	      (begin
+		(if (skk-context-candidate-window sc)
+		    (im-end-candidate id))
+		(skk-setup-child-context sc id)))))
     (if (not (skk-context-child-context sc))
 	(begin
 	  ;; Windowɽ򳫻Ϥ뤫
@@ -654,6 +666,8 @@
 	      (skk-make-string
 	       (skk-context-okuri sc) #t)
 	      (skk-context-nth sc))
+	     (if (> (skk-context-nth sc) 0)
+		 (skk-save-personal-dictionary))
 	     (if (skk-context-candidate-window sc)
 		 (im-end-candidate id))
 	     (skk-flush sc)
@@ -742,13 +756,21 @@
     (let* ((w (ja-wide (charcode->string key)))
 	   (id (context-id c))
 	   (sc (skk-current-context c)))
-      (if (skk-on-key key key-state)
-	  (begin
-	    (skk-flush sc)
-	    (skk-update-mode id sc))
-	  (if w
-	      (skk-commit sc id w)
-	      (skk-commit-raw sc id key key-state)))
+      (and
+       (if (skk-on-key key key-state)
+	   (begin
+	     (skk-flush sc)
+	     (skk-update-mode id sc)
+	     #f)
+	   #t)
+       (if (modifier-key-mask key-state)
+	   (begin
+	     (skk-commit-raw sc id key key-state)
+	     #f)
+	   #t)
+       (if w
+	   (skk-commit sc id w)
+	   (skk-commit-raw sc id key key-state)))
       ())))
 
 (define skk-push-key
