@@ -78,11 +78,11 @@ is_okuri(char *str)
 
 
 static int
-find_first(struct dic_info *di)
+find_first_line(struct dic_info *di)
 {
   char *s = di->addr;
   int off = 0;
-  while ( s[off] < di->size + s && s[off] == ';' ) {
+  while ( off < di->size && s[off] == ';' ) {
     int l = calc_line_len(&s[off]);
     off += l + 1;
   }
@@ -130,7 +130,7 @@ open_dic(const char *fn)
   di = (struct dic_info *)malloc(sizeof(struct dic_info));
   di->addr = addr;
   di->size = st.st_size;
-  di->first = find_first(di);
+  di->first = find_first_line(di);
   di->border = find_border(di);
   di->head.next = NULL;
   return di;
@@ -542,17 +542,29 @@ skk_get_nr_candidates(LISP head_, LISP okuri_head_, LISP okuri_)
   return flocons(n);
 }
 
-static
-reorder_candidate(struct skk_cand_array *ca, int nth)
+static void
+reorder_candidate(struct skk_cand_array *ca, char *str)
 {
-  if (ca && ca->nr_cands > nth) {
-    char *tmp;
-    int i;
-    tmp = ca->cands[nth];
-    for (i = nth; i > 0; i--) {
-      ca->cands[i] = ca->cands[i - 1];
+  int i;
+  int nth = 0;
+  char *tmp;
+  for (i = 0; i < ca->nr_cands; i++) {
+    if (!strcmp(str, ca->cands[i])) {
+      nth = i;
     }
-    ca->cands[0] = tmp;
+  }
+  if (nth == 0) {
+    return ;
+  }
+
+  tmp = ca->cands[nth];
+  for (i = nth; i > 0; i--) {
+    ca->cands[i] = ca->cands[i - 1];
+  }
+  ca->cands[0] = tmp;
+  /**/
+  if (nth >= ca->nr_real_cands) {
+    ca->nr_real_cands ++;
   }
 }
 
@@ -562,6 +574,7 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
 {
   int nth;
   struct skk_cand_array *ca;
+  char *str;
 
   nth = get_c_long(nth_);
   if (nth == 0) {
@@ -569,14 +582,33 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
   }
 
   ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+  if (!ca || ca->nr_cands <= nth) {
+    return NIL;
+  }
+
+  str = ca->cands[nth];
+  reorder_candidate(ca, str);
+
+  if (okuri_ != NIL) {
+    ca = find_cand_array_lisp(head_, okuri_head_, NIL);
+    if (!ca || ca->nr_cands <= nth) {
+      return NIL;
+    }
+    reorder_candidate(ca, str);
+  }
 
-  reorder_candidate(ca, nth);
   ca->line->need_save = 1;
 
   return NIL;
 }
 
-void
+static LISP
+skk_learn_word(LISP head_, LISP okuri_head_, LISP okuri_, LISP word_)
+{
+  return NIL;
+}
+
+static void
 parse_dic_line(char *line)
 {
   char *buf, *sep;
@@ -723,5 +755,6 @@ void uim_init_skk_dic()
   init_subr_4("skk-lib-get-nth-candidate", skk_get_nth_candidate);
   init_subr_3("skk-lib-get-nr-candidates", skk_get_nr_candidates);
   init_subr_4("skk-lib-commit-candidate", skk_commit_candidate);
+  init_subr_4("skk-lib-learn-word", skk_learn_word);
   init_subr_1("skk-lib-split-string", skk_split_string);
 }
