@@ -69,13 +69,13 @@ uim_helper_client_focus(uim_context uc, int flg)
   if(uim_fd < 0)
     return;
 
-  if(!uc || !uc->global_id || uc->global_id < 0)
+  if(!uc)
     return;
 
   if(flg == 0)
-    snprintf(uim_help_buf, BUFFER_SIZE, "focus_in:%d\n",  uc->global_id);
+    snprintf(uim_help_buf, BUFFER_SIZE, "focus_in\n");
   else
-    snprintf(uim_help_buf, BUFFER_SIZE, "focus_out:%d\n", uc->global_id);
+    snprintf(uim_help_buf, BUFFER_SIZE, "focus_out\n");
 
   uim_helper_send_message(uim_fd, uim_help_buf);
 }
@@ -90,26 +90,6 @@ uim_helper_client_get_prop_list(void)
 }
 
 
-int
-uim_helper_manage_context(uim_context uc)
-{
-  char buf[1024];
-  int i,rc;
-
-  uc->global_id = -1;
-
-
-  write(uim_fd, "new_context\n", strlen("new_context\n"));
-  rc = read(uim_fd, buf, sizeof(buf)-1);
-  buf[rc] = '\0';
-  sscanf(buf,
-	 "new_context:%d\n", &(uc->global_id));      
-  
-  printf("new context id:%d\n",(uc->global_id));
-  
-  return uc->global_id; 
-}
-
 void
 uim_helper_read_proc(void)
 {
