@@ -192,6 +192,18 @@ uim_set_prop_label_update_cb(uim_context uc,
 }
 
 
+void
+uim_prop_activate(uim_context uc, char *str)
+{
+  char buf[100];
+  
+  if(!str)
+    return;
+      
+  snprintf(buf, 99, "(prop-handler %d \"%s\")", uc->id, str);
+  uim_eval_string(uc, buf);
+}
+
 int
 uim_get_current_mode(uim_context uc)
 {
@@ -217,10 +229,15 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
 void
 uim_prop_list_update(uim_context uc)
 {
-  printf("prop list update\n");
   uc->prop_list_update_cb(uc, uc->propstr);
 }
 
+void
+uim_prop_label_update(uim_context uc)
+{
+  uc->prop_label_update_cb(uc, uc->proplabelstr);
+}
+
 
 static int
 can_convert_p(char *from, char *to)
